/*
 * Copyright (c) 2005 Sun Microsystems, Inc.  All rights reserved.  U.S.
 * Government Rights - Commercial software.  Government users are subject
 * to the Sun Microsystems, Inc. standard license agreement and
 * applicable provisions of the FAR and its supplements.  Use is subject
 * to license terms.
 *
 * This distribution may include materials developed by third parties.
 * Sun, Sun Microsystems, the Sun logo, Java and J2EE are trademarks
 * or registered trademarks of Sun Microsystems, Inc. in the U.S. and
 * other countries.
 *
 * Copyright (c) 2005 Sun Microsystems, Inc. Tous droits reserves.
 *
 * Droits du gouvernement americain, utilisateurs gouvernementaux - logiciel
 * commercial. Les utilisateurs gouvernementaux sont soumis au contrat de
 * licence standard de Sun Microsystems, Inc., ainsi qu'aux dispositions
 * en vigueur de la FAR (Federal Acquisition Regulations) et des
 * supplements a celles-ci.  Distribue par des licences qui en
 * restreignent l'utilisation.
 *
 * Cette distribution peut comprendre des composants developpes par des
 * tierces parties. Sun, Sun Microsystems, le logo Sun, Java et J2EE
 * sont des marques de fabrique ou des marques deposees de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;
import java.net.*;
import java.security.*;
import java.security.cert.X509Certificate;
import javax.security.auth.x500.X500PrivateCredential;
import java.util.*;
import java.io.*;
import org.freebxml.omar.client.xml.registry.util.JAXRUtility;

/**
 * The JAXRDelete class consists of a main method, a makeConnection
 * method, and an executeRemove method. This class finds and 
 * deletes the object specified by the string key ID argument.
 */
public class JAXRDelete {
    Connection connection = null;
    RegistryService rs = null;

    public JAXRDelete() {
    }

    public static void main(String[] args) {
        ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");
        javax.xml.registry.infomodel.Key key = null;

        String queryURL = bundle.getString("query.url");
        String publishURL = bundle.getString("publish.url");

        if (args.length < 1) {
            System.out.println("Argument required: " + "urn");
            System.exit(1);
        }

        String urn = new String(args[0]);
        System.out.println("URN is " + urn);

        JAXRDelete jd = new JAXRDelete();

        jd.makeConnection(queryURL, publishURL);
        
        key = jd.createObjKey(urn);

        if (key != null) {
            jd.executeRemove(key);
        } else {
            System.out.println("URN not created, nothing to remove");
        }
    }

    /**
     * Establishes a connection to a registry.
     *
     * @param queryUrl        the URL of the query registry
     * @param publishUrl      the URL of the publish registry
     */
    public void makeConnection(String queryUrl, String publishUrl) {

        ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");

        /*
         * Define connection configuration properties.
         * To delete, you need both the query URL and the
         * publish URL.
         */
        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL", queryUrl);
        props.setProperty("javax.xml.registry.lifeCycleManagerURL", publishUrl);

        try {
            ConnectionFactory factory = JAXRUtility.getConnectionFactory();
            factory.setProperties(props);
            connection = factory.createConnection();
            System.out.println("Created connection to registry");

            // Get authorization from the registry

            String keystorePath = bundle.getString("security.keystorePath");
            String storepass = bundle.getString("security.storepass");
            String alias = bundle.getString("security.alias");
            String keypass = bundle.getString("security.keypass");

            //Set credentials = new HashSet();
            //credentials = 
            //    getCredentialsFromKeystore(keystorePath, storepass, alias, keypass);
            //connection.setCredentials(credentials);
            //System.out.println("Established security credentials");
        } catch (Exception e) {
            e.printStackTrace();

            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                }
            }
        }
    }

    /**
     * Creates a Key object from the user-supplied string.
     *
     * @param keyStr        the key of the published object
     *
     * @return        the key of the object found
     */
    public javax.xml.registry.infomodel.Key createObjKey(String keyStr) {
        BusinessLifeCycleManager blcm = null;
        javax.xml.registry.infomodel.Key objKey = null;

        try {
            rs = connection.getRegistryService();
            blcm = rs.getBusinessLifeCycleManager();
            System.out.println("Got registry service and " +
                "life cycle manager");

            objKey = blcm.createKey(keyStr);
        } catch (Exception e) {
            e.printStackTrace();

            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                    System.err.println("Connection close failed");
                }
            }
        }

        return objKey;
    }

    /**
     * Removes the registry object with the specified key value.
     *
     * @param key        the Key of the object
     */
    public void executeRemove(javax.xml.registry.infomodel.Key key) {
        BusinessLifeCycleManager blcm = null;

        try {
            blcm = rs.getBusinessLifeCycleManager();

            String id = key.getId();
            System.out.println("Deleting object with id " + id);

            Collection keys = new ArrayList();
            keys.add(key);

            BulkResponse response = blcm.deleteObjects(keys);
            Collection exceptions = response.getExceptions();

            if (exceptions == null) {
                System.out.println("Object deleted");

                Collection retKeys = response.getCollection();
                Iterator keyIter = retKeys.iterator();
                javax.xml.registry.infomodel.Key objKey = null;

                if (keyIter.hasNext()) {
                    objKey = (javax.xml.registry.infomodel.Key) keyIter.next();
                    id = objKey.getId();
                    System.out.println("Object URN was " + id);
                }
            } else {
                Iterator excIter = exceptions.iterator();
                Exception exception = null;

                while (excIter.hasNext()) {
                    exception = (Exception) excIter.next();
                    System.err.println("Exception on delete: " +
                        exception.toString());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // At end, close connection to registry
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                }
            }
        }
    }

    /**
     * From test/org/freebxml/omar/client/common/ClientTest.java.
     *
     * Obtains a set of credentials using a certificate.
     *
     * @param keystorePath   location of client keystore
     * @param storepass      keystore password
     * @param alias          user name
     * @param keypass        user password
     *
     * @return   a set of credentials
     */
 /*
    private Set getCredentialsFromKeystore(String keystorePath, 
            String storepass, String alias, String keypass) throws Exception {
        HashSet credentials = new HashSet();
        KeyStore keyStore = KeyStore.getInstance("JKS");
        keyStore.load(new BufferedInputStream(new FileInputStream(keystorePath)), 
            storepass.toCharArray());
        X509Certificate cert = (X509Certificate) keyStore.getCertificate(alias);
        PrivateKey privateKey = 
            (PrivateKey) keyStore.getKey(alias, keypass.toCharArray());
        credentials.add(new X500PrivateCredential(cert, privateKey, alias));
        return credentials;
    }
 */
}
