/*
 * Copyright (c) 2005 Sun Microsystems, Inc.  All rights reserved.  U.S.
 * Government Rights - Commercial software.  Government users are subject
 * to the Sun Microsystems, Inc. standard license agreement and
 * applicable provisions of the FAR and its supplements.  Use is subject
 * to license terms.
 *
 * This distribution may include materials developed by third parties.
 * Sun, Sun Microsystems, the Sun logo, Java and J2EE are trademarks
 * or registered trademarks of Sun Microsystems, Inc. in the U.S. and
 * other countries.
 *
 * Copyright (c) 2005 Sun Microsystems, Inc. Tous droits reserves.
 *
 * Droits du gouvernement americain, utilisateurs gouvernementaux - logiciel
 * commercial. Les utilisateurs gouvernementaux sont soumis au contrat de
 * licence standard de Sun Microsystems, Inc., ainsi qu'aux dispositions
 * en vigueur de la FAR (Federal Acquisition Regulations) et des
 * supplements a celles-ci.  Distribue par des licences qui en
 * restreignent l'utilisation.
 *
 * Cette distribution peut comprendre des composants developpes par des
 * tierces parties. Sun, Sun Microsystems, le logo Sun, Java et J2EE
 * sont des marques de fabrique ou des marques deposees de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;
import java.net.*;
import java.security.*;
import java.security.cert.X509Certificate;
import javax.security.auth.x500.X500PrivateCredential;
import java.util.*;
import java.io.*;
import org.freebxml.omar.client.xml.registry.util.JAXRUtility;

/**
 * The JAXRPublishOrg class consists of a main method, a makeConnection
 * method, and an executePublish method. It creates an organization and 
 * publishes it to a registry.
 *
 * ECE355 Change - Oct. 2005 - Remove service creation.
 */
public class JAXRPublishOrg {
    Connection connection = null;

    public JAXRPublishOrg() {
    }

    public static void main(String[] args) {
        ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");

        String queryURL = bundle.getString("query.url");
        String publishURL = bundle.getString("publish.url");

        JAXRPublishOrg jp = new JAXRPublishOrg();

        jp.makeConnection(queryURL, publishURL);

        jp.executePublish();
    }

    /**
     * Establishes a connection to a registry.
     *
     * @param queryUrl        the URL of the query registry
     * @param publishUrl        the URL of the publish registry
     */
    public void makeConnection(String queryUrl, String publishUrl) {

        ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");

        /*
         * Define connection configuration properties.
         * To publish, you need both the query URL and the
         * publish URL.
         */
        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL", queryUrl);
        props.setProperty("javax.xml.registry.lifeCycleManagerURL", publishUrl);

        try {
            ConnectionFactory factory = JAXRUtility.getConnectionFactory();
            factory.setProperties(props);
            connection = factory.createConnection();
            System.out.println("Created connection to registry");

            // Get authorization from the registry

            String keystorePath = bundle.getString("security.keystorePath");
            String storepass = bundle.getString("security.storepass");
            String alias = bundle.getString("security.alias");
            String keypass = bundle.getString("security.keypass");

            Set credentials = new HashSet();
            //credentials = 
            //    getCredentialsFromKeystore(keystorePath, storepass, alias, keypass);
            //connection.setCredentials(credentials);
            //System.out.println("Established security credentials");
        } catch (Exception e) {
            e.printStackTrace();

            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                }
            }
        }
    }

    /**
     * Creates an organization, its primary contact, its classification, 
     * and its services, and saves it to the registry.
     */
    public void executePublish() {
        RegistryService rs = null;
        BusinessLifeCycleManager blcm = null;
        BusinessQueryManager bqm = null;

        try {
            rs = connection.getRegistryService();
            blcm = rs.getBusinessLifeCycleManager();
            bqm = rs.getBusinessQueryManager();
            System.out.println("Got registry service, query " +
                "manager, and life cycle manager");

            ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");

            // Create organization name and description
            Organization org =
                blcm.createOrganization(bundle.getString("org.name"));
            String orgId = org.getKey().getId();
            System.out.println("Organization URN is " + orgId);

            InternationalString is =
                blcm.createInternationalString(bundle.getString(
                        "org.description"));
            org.setDescription(is);

            // create postal address for organization
            String streetNumber = 
                bundle.getString("postal.streetNumber");
            String street = bundle.getString("postal.street");
            String city = bundle.getString("postal.city");
            String state = bundle.getString("postal.state");
            String country = bundle.getString("postal.country");
            String postalCode = bundle.getString("postal.postalCode");
            String type = bundle.getString("postal.type");
            PostalAddress postAddr =
                blcm.createPostalAddress(streetNumber, street, city, state,
                    country, postalCode, type);
            org.setPostalAddress(postAddr);

            // create telephone number for organization
            TelephoneNumber tNum = blcm.createTelephoneNumber();
            tNum.setCountryCode(bundle.getString("phone.countrycode"));
            tNum.setAreaCode(bundle.getString("phone.areacode"));
            tNum.setNumber(bundle.getString("phone.number"));
            tNum.setType(bundle.getString("phone.type"));
            Collection tNums = new ArrayList();
            tNums.add(tNum);
            org.setTelephoneNumbers(tNums);
            
            // Create primary contact, set name
            User primaryContact = blcm.createUser();
            String userId = primaryContact.getKey().getId();
            System.out.println("User URN is " + userId);

            PersonName pName =
                blcm.createPersonName(bundle.getString("person.firstname"),
                   bundle.getString("person.middlename"),
                   bundle.getString("person.lastname"));
            primaryContact.setPersonName(pName);

            // Set primary contact phone number
            TelephoneNumber pctNum = blcm.createTelephoneNumber();
            pctNum.setCountryCode(bundle.getString("pcphone.countrycode"));
            pctNum.setAreaCode(bundle.getString("pcphone.areacode"));
            pctNum.setNumber(bundle.getString("pcphone.number"));
            pctNum.setType(bundle.getString("pcphone.type"));
            Collection phoneNums = new ArrayList();
            phoneNums.add(pctNum);
            primaryContact.setTelephoneNumbers(phoneNums);

            // Set primary contact email address
            EmailAddress emailAddress =
                blcm.createEmailAddress(bundle.getString("email.address"));
            emailAddress.setType(bundle.getString("email.type"));
            Collection emailAddresses = new ArrayList();
            emailAddresses.add(emailAddress);
            primaryContact.setEmailAddresses(emailAddresses);
            
            URL pcUrl = new URL("http://TheCoffeeBreak.com/JaneMDoe.html");
            primaryContact.setUrl(pcUrl);

            // Set primary contact for organization
            org.setPrimaryContact(primaryContact);

            // Set classification scheme to ISO 3166
            String schemeName = bundle.getString("classification.scheme");
            ClassificationScheme cScheme = 
                bqm.findClassificationSchemeByName(null, schemeName);
            
            if (cScheme != null) {
                // Create and add classification
                Classification classification = 
                    blcm.createClassification(cScheme, 
                        bundle.getString("classification.name"), 	
                        bundle.getString("classification.value"));
                org.addClassification(classification);
            }


         
           

 
            // Add organization and submit to registry
            // Retrieve key if successful
            Collection orgs = new ArrayList();
            orgs.add(org);

            BulkResponse response = blcm.saveOrganizations(orgs);
            Collection exceptions = response.getExceptions();

            if (exceptions == null) {
                System.out.println("Organization saved");
            } else {
                Iterator excIter = exceptions.iterator();
                Exception exception = null;

                while (excIter.hasNext()) {
                    exception = (Exception) excIter.next();
                    System.err.println("Exception on save: " +
                        exception.toString());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // At end, close connection to registry
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                }
            }
        }
    }

    /**
     * From test/org/freebxml/omar/client/common/ClientTest.java.
     *
     * Obtains a set of credentials using a certificate.
     *
     * @param keystorePath   location of client keystore
     * @param storepass      keystore password
     * @param alias          user name
     * @param keypass        user password
     *
     * @return   a set of credentials
     */
    
   
/*   
    private Set getCredentialsFromKeystore(String keystorePath, 
            String storepass, String alias, String keypass) throws Exception {
        HashSet credentials = new HashSet();
        KeyStore keyStore = KeyStore.getInstance("JKS");
        keyStore.load(new BufferedInputStream(new FileInputStream(keystorePath)), 
            storepass.toCharArray());
        X509Certificate cert = (X509Certificate) keyStore.getCertificate(alias);
        PrivateKey privateKey = 
            (PrivateKey) keyStore.getKey(alias, keypass.toCharArray());
        credentials.add(new X500PrivateCredential(cert, privateKey, alias));
        return credentials;
    }
*/
}

