/*
 * Copyright (c) 2005 Sun Microsystems, Inc.  All rights reserved.  U.S.
 * Government Rights - Commercial software.  Government users are subject
 * to the Sun Microsystems, Inc. standard license agreement and
 * applicable provisions of the FAR and its supplements.  Use is subject
 * to license terms.
 *
 * This distribution may include materials developed by third parties.
 * Sun, Sun Microsystems, the Sun logo, Java and J2EE are trademarks
 * or registered trademarks of Sun Microsystems, Inc. in the U.S. and
 * other countries.
 *
 * Copyright (c) 2005 Sun Microsystems, Inc. Tous droits reserves.
 *
 * Droits du gouvernement americain, utilisateurs gouvernementaux - logiciel
 * commercial. Les utilisateurs gouvernementaux sont soumis au contrat de
 * licence standard de Sun Microsystems, Inc., ainsi qu'aux dispositions
 * en vigueur de la FAR (Federal Acquisition Regulations) et des
 * supplements a celles-ci.  Distribue par des licences qui en
 * restreignent l'utilisation.
 *
 * Cette distribution peut comprendre des composants developpes par des
 * tierces parties. Sun, Sun Microsystems, le logo Sun, Java et J2EE
 * sont des marques de fabrique ou des marques deposees de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;
import java.net.*;
import java.security.*;
import java.security.cert.X509Certificate;
import javax.security.auth.x500.X500PrivateCredential;
import java.util.*;
import java.io.*;
import javax.activation.*;
import org.freebxml.omar.client.xml.registry.BusinessQueryManagerImpl;
import org.freebxml.omar.client.xml.registry.infomodel.ExtrinsicObjectImpl;
import org.freebxml.omar.client.xml.registry.util.JAXRUtility;

/**
 * The JAXRPublishService class consists of a main method, a
 * makeConnection method, and an executePublish method.
 * It publishes a service to the registry and saves the associated WSDL file
 * to the repository.
 */
public class JAXRPublishService {
    Connection connection = null;

    public JAXRPublishService() {
    }

    public static void main(String[] args) {
        ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");

        String queryURL = bundle.getString("query.url");
        String publishURL = bundle.getString("publish.url");
        System.out.println("Query URL is " + queryURL);
        System.out.println("Publish URL is " + publishURL);


        if (args.length < 1) {
            System.out.println("One argument required: " +
                "organizationKey");
            System.exit(1);
        }
        String orgKey = new String(args[0]);
        System.out.println("org key  is " + orgKey);

       

        JAXRPublishService jps = new JAXRPublishService();

        jps.makeConnection(queryURL, publishURL);

        jps.executePublish(orgKey);
    }

    /**
     * Establishes a connection to a registry.
     *
     * @param queryUrl        the URL of the query registry
     * @param publishUrl      the URL of the publish registry
     */
    public void makeConnection(String queryUrl, String publishUrl) {

        ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");

        /*
         * Define connection configuration properties.
         * To publish, you need both the query URL and the
         * publish URL.
         */
        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL", queryUrl);
        props.setProperty("javax.xml.registry.lifeCycleManagerURL", publishUrl);

        try {
            ConnectionFactory factory = JAXRUtility.getConnectionFactory();
            factory.setProperties(props);
            connection = factory.createConnection();
            System.out.println("Created connection to registry");

            // Get authorization from the registry

            String keystorePath = bundle.getString("security.keystorePath");
            String storepass = bundle.getString("security.storepass");
            String alias = bundle.getString("security.alias");
            String keypass = bundle.getString("security.keypass");

           //Set credentials = new HashSet();
           //credentials = 
           //    getCredentialsFromKeystore(keystorePath, storepass, alias, keypass);
           // connection.setCredentials(credentials);
           // System.out.println("Established security credentials");
        } catch (Exception e) {
            e.printStackTrace();

            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                }
            }
        }
    }

    /**
     * Creates a service and saves it to the registry, also putting the
     * WSDL file for the service in the repository.
     */
    public void executePublish(String orgKey) {
        RegistryService rs = null;
        BusinessLifeCycleManager blcm = null;
        BusinessQueryManagerImpl bqm = null;
        ResourceBundle bundle = ResourceBundle.getBundle("JAXRExamples");

        try {
            rs = connection.getRegistryService();
            blcm = rs.getBusinessLifeCycleManager();
            bqm = (BusinessQueryManagerImpl) rs.getBusinessQueryManager();
            System.out.println("Got registry service, query " +
                "manager, and life cycle manager");
            
            // Create services and service
            Collection services = new ArrayList();
            Service service = 
                blcm.createService(bundle.getString("hello.service.name"));
            InternationalString is = 
                blcm.createInternationalString(bundle.getString(
                        "hello.service.description"));
            service.setDescription(is);
            
            // Find organization providing this service
            RegistryObject obj1 = bqm.getRegistryObject(orgKey);
            Concept objType = obj1.getObjectType();
            Organization org  = null ;

            if (obj1 != null & objType.getValue().equals("Organization") ) {
                          
                System.out.println("Loooking for Org - Object type is " + objType.getValue());
                System.out.println("Loooking for Org -Object name is " + obj1.getName());
                System.out.println("Loooking for Org -Object description is " + 
                    obj1.getDescription());
               org = (Organization)(obj1);
               service.setProvidingOrganization((Organization)(obj1));
                
            } else {
                System.err.println("Organization not found for " + orgKey);
                return ;
            }


            
            // Create service bindings
            Collection serviceBindings = new ArrayList();
            ServiceBinding binding = blcm.createServiceBinding();
            is = blcm.createInternationalString(bundle.getString(
                        "hello.svcbnd.name"));
            binding.setName(is);
            is = blcm.createInternationalString(bundle.getString(
                        "hello.svcbnd.description"));
            binding.setDescription(is);
            // allow us to publish a fictitious URI without an error
            binding.setValidateURI(false);
            binding.setAccessURI(bundle.getString("hello.svcbnd.uri"));
            System.out.println("Set AccessURI");
            
            // Create specification link
            SpecificationLink specLink = blcm.createSpecificationLink();
            is = blcm.createInternationalString(bundle.getString(
                        "hello.speclink.name"));
            specLink.setName(is);
            is = blcm.createInternationalString(bundle.getString(
                        "hello.speclink.description"));
            specLink.setDescription(is);
            
            String filename = bundle.getString("hello.filename");
            File repositoryItemFile = new File(filename);
            DataHandler repositoryItem = 
                new DataHandler(new FileDataSource(repositoryItemFile));
            ExtrinsicObject eo = 
                blcm.createExtrinsicObject(repositoryItem);
            System.out.println("Created ExtrinsicObject from " + filename);
            is = blcm.createInternationalString(bundle.getString(
                    "hello.exobj.name"));
            eo.setName(is);
            
            // Find WSDL concept for extrinsic object
            // This concept has a value but no name: search by ID
            String conceptId = bundle.getString("hello.concept.id");
            Concept objectTypeConcept = 
                (Concept) bqm.getRegistryObject(conceptId);
            System.out.println("Concept value: " + 
                objectTypeConcept.getValue());
            ClassificationScheme cScheme = 
                objectTypeConcept.getClassificationScheme();
            System.out.println("Classification scheme is " +
                cScheme.getName());

            /*
			 * Cast to implementation-specific class; setObjectType is extension
			 * of JAXR 1.0
			 */
            ((ExtrinsicObjectImpl)eo).setObjectType(objectTypeConcept);
            System.out.println("Set object type to " + objectTypeConcept.getValue());
            
            // Set MIME type appropriately
            eo.setMimeType("text/xml");
            System.out.println("Set MIME type to " + eo.getMimeType());

            specLink.setSpecificationObject(eo);
            System.out.println("Set SpecificationObject for SpecificationLink");            

            //**** ECE355 Change *******************/
            // Add wsdl info as external link to binding

            is = blcm.createInternationalString(bundle.getString(
                        "hello.wsdl.desc"));           
            String uri = bundle.getString("hello.wsdl.uri");  
  
            ExternalLink wsdlLink = blcm.createExternalLink(uri,is);

            binding.addExternalLink(wsdlLink);
               
            //**** End of ECE355 Change *******************/

            binding.addSpecificationLink(specLink);
            
            serviceBindings.add(binding);

            // Add service bindings to service
            service.addServiceBindings(serviceBindings);
            
            // Add service to services
             services.add(service);
             org.addServices(services);

            
            // Add organization and submit to registry
            // Retrieve key if successful
            Collection orgs = new ArrayList();
            orgs.add(org);

            BulkResponse response = blcm.saveOrganizations(orgs);
            Collection exceptions = response.getExceptions();
            
            if (exceptions == null) {
                System.out.println("Service saved - key:" + service.getKey());
            } else {
                Iterator excIter = exceptions.iterator();
                Exception exception = null;

                while (excIter.hasNext()) {
                    exception = (Exception) excIter.next();
                    System.err.println("Exception on save: " +
                        exception.toString());
                }
            }
           

           
            
             
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // At end, close connection to registry
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                }
            }
        }
    }

    /**
     * From test/org/freebxml/omar/client/common/ClientTest.java.
     *
     * Obtains a set of credentials using a certificate.
     *
     * @param keystorePath   location of client keystore
     * @param storepass      keystore password
     * @param alias          user name
     * @param keypass        user password
     *
     * @return   a set of credentials
     */
/*    
    private Set getCredentialsFromKeystore(String keystorePath, 
            String storepass, String alias, String keypass) throws Exception {
        HashSet credentials = new HashSet();
        KeyStore keyStore = KeyStore.getInstance("JKS");
        keyStore.load(new BufferedInputStream(new FileInputStream(keystorePath)), 
            storepass.toCharArray());
        X509Certificate cert = (X509Certificate) keyStore.getCertificate(alias);
        PrivateKey privateKey = 
            (PrivateKey) keyStore.getKey(alias, keypass.toCharArray());
        credentials.add(new X500PrivateCredential(cert, privateKey, alias));
        return credentials;
    }
*/    
}

