/*
 * Created on 26-Oct-2005
 * @author talguind
 *
 * Finds services for organization
 *
 * The WebServicesQuery class finds services offered by an organization.
 * It takes as a parameter a name pattern, and finds the services offered by 
 * organizations that match this name pattern.
 * 
 * The information on the services found is returned as a collection of 
 * ServiceInfo objects. Only services with wsdl info are included in the results.
 * 
 */
 

import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;

import java.net.*;
import java.util.*;

import org.freebxml.omar.client.xml.registry.BusinessQueryManagerImpl;
import org.freebxml.omar.client.xml.registry.util.JAXRUtility;


public class  WebServicesQuery {
    Connection connection = null;
    private static WebServicesQuery wsqInstance ;
    String queryURL ;
    String publishURL ;
    static final String ResourceBundleFile = "JAXRExamples" ;
    Collection services = new Vector() ;
    Collection serviceInfoColl = new Vector() ;
    
    
    
    
    
    protected  WebServicesQuery() {}
    
    public static WebServicesQuery getInstance(){
    	if (wsqInstance == null) 
    		wsqInstance =  new WebServicesQuery();
    	return wsqInstance ;
    }

    private  void init(){
        ResourceBundle bundle =
           ResourceBundle.getBundle(ResourceBundleFile);

        queryURL = bundle.getString("query.url");
        publishURL = bundle.getString("publish.url");
    }  
    
    public Collection findServicesForOrg(String orgName){
    	init();
    	makeConnection();
    	executeQuery(orgName);
    	return serviceInfoColl;
    
    }
    
    /**
     * Establishes a connection to a registry.
     *
     * @param queryUrl   the URL of the query registry
     * @param publishUrl the URL of the publish registry
     */
    public void makeConnection() {

        /* 
         * Define connection configuration properties. 
         * For simple queries, you need the query URL.
         * To use a life cycle manager, you need the publish URL.
         */
        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL",
            queryURL);

        try {
            ConnectionFactory factory = JAXRUtility.getConnectionFactory();
            factory.setProperties(props);
            connection = factory.createConnection();
            System.out.println("Created connection to registry");
        } catch (Exception e) {
            e.printStackTrace();
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {}
            }
        }
    }

    /**
     * Searches for services and  wsdl info info for an organization.
     * 
     * If more than one organization matches the given name
     * pattern(in qString) , it returns services for all these matching organizations.
     * 
     * Services without wsdl info are ignored in this query.
     */
    public void executeQuery(String qString) {
    	String typeString = "Organization";
    	RegistryService rs = null;
    	BusinessQueryManagerImpl bqm = null;
    	
    	try {
    		// Get registry service and managers
    		rs = connection.getRegistryService();
    		bqm = (BusinessQueryManagerImpl) rs.getBusinessQueryManager();
    		System.out.println("Got registry service and query manager");
    		
    		System.out.println("Searching for objects of type " + 
    				typeString);
    		System.out.println("Searching for name pattern " + 
    				qString);
    		
    		
    		/*
    		 * Query is not case-sensitive but cannot use wildcards
    		 * No FindQualifiers specified, so as to find objects that
    		 * have no names
    		 */
    		
    		
    		// Define find qualifiers and name patterns
    		Collection findQualifiers = new ArrayList();
    		findQualifiers.add(FindQualifier.SORT_BY_NAME_ASC);
    		
    		Collection namePatterns = new ArrayList();
    		namePatterns.add("%" + qString + "%");
    		
    		BulkResponse response =
    			bqm.findOrganizations(findQualifiers, namePatterns, null, null,
    					null, null);
    		
    		Collection objects = response.getCollection();
    		
    		// Display information about the objects found
    		Iterator orgIter = objects.iterator();
    		if (!(orgIter.hasNext())) {
    			System.out.println("No objects found");
    			return ;
    			
    		} else {
    			// Look  for services for the organization
    			// We look for associations of type "OffersService".
                // This is a workaround to the problem that we experience when we use org.getServices() -
                // org.getServices() does not return the services linked to an organization.
               
    			while (orgIter.hasNext()) {
    				Organization org = (Organization) orgIter.next();
   				
    				
    				Collection associations = org.getAssociations();
   				    // System.out.println("*******Number of Associations for :"  + org.getName() + " is " + associations.size());
    				Iterator assocIter = associations.iterator();
    				while (assocIter.hasNext()) {
    					Association assoc = (Association)assocIter.next();
                        String assocType = assoc.getAssociationType().toString();

                        if ((assocType.trim()).equals("OffersService")){
                            services.add(assoc.getTargetObject());
    						//System.out.println("******Assoc description:"  + assoc.getDescription() );
    						//System.out.println("******Assoc type:"  + assoc.getAssociationType() );
    						//System.out.println("******Assoc target object:"  + assoc.getTargetObject().getName() );
    					}
    					
    					
    				}
    				
    				//System.out.println("***********************Total OffersService Associations:"  + services.size() );
    				
    			}
    			// Now that we have the services, lets get the details for these services
    			
    			Iterator svcIter = services.iterator();
    			//System.out.println("Number of Services :" + services.size());
    			if (services!= null) {
    				for (Iterator iterator_sv = services.iterator(); iterator_sv.hasNext();) {	
    					Service service  = (Service)iterator_sv.next();
    					
    					
    					// Get Service Binding for service
    					Collection collServBinds = service.getServiceBindings();
    					//System.out.println("Number of Service Bindings:" + collServBinds.size());
    					if (collServBinds!= null) {
    						for (Iterator iterator_sb = collServBinds.iterator(); iterator_sb.hasNext();) {	
    							ServiceBinding sb = (ServiceBinding)iterator_sb.next();
    							
    							Collection extLinks = 	sb.getExternalLinks();
    							//System.out.println("Number of ExternalLinks:" + extLinks.size());
    							if (extLinks!= null) {
    								for (Iterator iterator_el = extLinks.iterator(); iterator_el.hasNext();) {
    									ExternalLink externalLink = (ExternalLink)iterator_el.next();
    									//System.out.println("External Links: ");
    									//System.out.println("WSDL description  " + externalLink.getDescription());
    									//System.out.println("WSDL url " + externalLink.getExternalURI());
    									
    									ServiceInfo serviceInfo = new ServiceInfo();
    									serviceInfo.setProvidingOrganization(service.getProvidingOrganization().getName());
    									serviceInfo.setName(service.getDescription());
    									serviceInfo.setDescription(service.getDescription());
    									serviceInfo.setEndPoint(sb.getAccessURI());
    									serviceInfo.setWsdluri(externalLink.getExternalURI());
    									serviceInfoColl.add(serviceInfo);
    								}
    							}
    							
    							
    						}
    					}
    					
    				}
    			}
    			
    			// Print spacer between objects
    			//System.out.println(" --- ");
    			
    		}
    		
    		
    	}
    	catch (RegistryException re) {
    		// This will never happen now since we are only looking for Organizations
    		System.err.println("\"" + typeString + "\" is not a valid " +
    		"object type. Verify spelling.");
    	} catch (Exception e) {
    		e.printStackTrace();
    	} finally {
    		// At end, close connection to registry
    		if (connection != null) {
    			try {
    				connection.close();
    			} catch (JAXRException je) {
    			}
    		}
    	}
    }

    /**
     * Returns the name value for a registry object.
     *
     * @param ro	a RegistryObject
     * @return		the String value
     */
    private String getName(RegistryObject ro) 
        throws JAXRException {

        try {
            return ro.getName().getValue();
        } catch (NullPointerException npe) {
            return "No Name";
        }
    }
    
    /**
     * Returns the description value for a registry object.
     *
     * @param ro	a RegistryObject
     * @return		the String value
     */
    private String getDescription(RegistryObject ro) 
        throws JAXRException {

        try {
            return ro.getDescription().getValue();
        } catch (NullPointerException npe) {
            return "No Description";
        }
    }
    
    /**
     * Returns the key id value for a registry object.
     *
     * @param ro	a RegistryObject
     * @return		the String value
     */
    private String getKey(RegistryObject ro) 
        throws JAXRException {

        try {
            return ro.getKey().getId();
        } catch (NullPointerException npe) {
            return "No Key";
        }
    }
}