/*
 * Created on Oct 18, 2004
 * (ECE355 - Log Feature Solution.)
 */

/**
* Class for writing a log of attempted calls.
* It provides methods to initiate writing to the log file, write 
* each attempted call to  the log  file and end logging.
* Since the class follows the singleton pattern, any instance of 
*  should be created with the method getInstance().
*/

package net.java.sip.communicator.common;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * @author talguind
 *
 */
public class CallLoggerText {
	/** 
	 * Unique instance of this class(as in Singleton Pattern)
	 */
	private static CallLogger instance ;
	 /** First field in the attempted calls log file,
     * identifiying the subscriber.
     */
	private String callingSubscriber;
	/**
	 * Second field in the attempted calls log file
     *     providing the time of the call.
	 */
	private String time;
	/** Third field   in the attempted calls log file,
    * describing the final status of the call: succesful, busy, cancelled.
    */
	private String replyStatus;
	
	private final static String newline = "\n";
	private File fileName = new File("callslog.txt");
	private FileWriter fw;
	
	/**
    * This method returns a single instance of the class. 
    * It ensures that only one instance of the class exists at any 
    * one time so if there is already an instance of the class it
    * returns a reference to the existing instance.
    *
    * @return    new or existing instance of this class.
    */

	public static synchronized CallLogger getInstance(){
		if (instance == null) instance = new CallLogger();
		return instance ;
		
	}
	/**
	 * Opens the log file and writes headings.
	 *
	 */
	public void init(){
		// Open file
		
		try {
			// Append to existing file or create file if file does not exist.
			fw = new FileWriter(fileName,true);
			// Write headings
			String s = "Calling Subscriber         Time                      Reply Status" ;
			fw.write(newline);
			fw.write(s);	
			fw.write(newline);
			fw.write(newline);
				
		}
			
	  catch (IOException ie){
	  	System.out.println("Can not open call's logfile");
	  }
	  }
	
	/**
	 * Closes the log file.
	 *
	 */
	public void end(){
		try {
			fw.close();
		}
		catch (IOException ie){
			System.out.println("Can not close call's log file");
		}
	}
	
	/**
	 *  Writes to the log file the subscriber, time and 
	 *  status for a call. 
	 *  Instance variables callingSunscriber, time and replyStatus must 
     *  be set previous to calling this method.
     */
	public void writeCallLog() {
		
		String s = callingSubscriber + "          "
			+ time + "        " + replyStatus  ;
		
		try {
			fw.write(s);
			fw.write(newline);
		}
		catch (IOException ie){
			System.out.println("Can not write to  call's log file");
		}
		finally
		{
			setCallingSubscriber("");
			setReplyStatus("");
			setTime("");
		};
	}
	    

	/**
	 * @return Returns the callingSubscriber.
	 */
	public String getCallingSubscriber() {
		return callingSubscriber;
	}
	/**
	 * @param callingSubscriber The callingSubscriber to set.
	 */
	public void setCallingSubscriber(String callingSubscriber) {
		this.callingSubscriber = callingSubscriber;
	}
	/**
	 * @return Returns the replyStatus.
	 */
	public String getReplyStatus() {
		return replyStatus;
	}
	/**
	 * @param replyStatus The replyStatus to set.
	 */
	public void setReplyStatus(String replyStatus) {
		this.replyStatus = replyStatus;
	}
	/**
	 * @return Returns the time.
	 */
	public String getTime() {
		return time;
	}
	/**
	 * @param time The time to set.
	 */
	public void setTime(String time) {
		this.time = time;
	}
}
